@@ -346,7 +346,7 @@ class Solution(object):
         elif file_format == 'ascii': 
             from clawpack.pyclaw import io
             read_func = io.ascii.read
-        elif file_format=='hdf': 
+        elif file_format in ('hdf','hdf5'): 
             from clawpack.pyclaw import io
             read_func = io.hdf5.read
         else:
