@@ -2,12 +2,12 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
 ! ============================================================================
 !  Solves transverse Riemann problem for the multilayer shallow water 
 !  equations in 2D with topography and wind forcing:
-!    (h_1)_t + (h_1 u_1)_x + (h_1 v_1)_y = 0
-!    (h_1 u_1)_t + (h_1 u_1^2 + 1/2 g h_1^2)_x + (h_1 u_1 v_1)_y = -gh_1(r(h_2)_x + B_x)
-!    (h_1 v_1)_t + (h_1 u_1 v_1)_x + (h_1 v_1^2 + 1/2 g h_1^2)_y = -gh_1(r(h_2)_y + B_y)
-!    (h_2)_t + (h_2 u_2)_x + (h_2 v_2)_y = 0
-!    (h_2 u_2)_t + (h_2 u_2^2 + 1/2 g h_2^2)_x + (h_2 u_2 v_2)_y = -gh_2(h_1 + B)_x + Tau |W| W_x
-!    (h_2 v_2)_t + (h_2 u_2 v_2)_x + (h_2 v_2^2 + 1/2 g h_2^2)_y = -gh_2(h_1 + B)_y + Tau |W| W_y
+!    (rho_1 h_1)_t + (rho_1 h_1 u_1)_x + (rho_1 h_1 v_1)_y = 0
+!    (rho_1 h_1 u_1)_t + (rho_1 h_1 u_1^2 + 1/2 g rho_1 h_1^2)_x + (rho_1 h_1 u_1 v_1)_y = -g rho_1 h_1(r(h_2)_x + B_x)
+!    (rho_1 h_1 v_1)_t + (rho_1 h_1 u_1 v_1)_x + (rho_1 h_1 v_1^2 + 1/2 g rho_1 h_1^2)_y = -g rho_1 h_1(r(h_2)_y + B_y)
+!    (rho_2 h_2)_t + (rho_2 h_2 u_2)_x + (rho_2 h_2 v_2)_y = 0
+!    (rho_2 h_2 u_2)_t + (rho_2 h_2 u_2^2 + 1/2 g rho_2 h_2^2)_x + (rho_2 h_2 u_2 v_2)_y = -g rho_2 h_2(h_1 + B)_x
+!    (rho_2 h_2 v_2)_t + (rho_2 h_2 u_2 v_2)_x + (rho_2 h_2 v_2^2 + 1/2 g rho_2 h_2^2)_y = -g rho_2 h_2(h_1 + B)_y
 !
 !  On input, ql contains the state vector at the left edge of each cell and qr
 !  contains the state vector at the right edge of each cell
@@ -23,11 +23,18 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
 !  if imp == 1 then the split is in the negative direction, imp == 2 positive
 !
 !  Kyle T. Mandli (10-13-2010)
-! ============================================================================
+! ==============================================================================
+!  Note that this version does not work with PyClaw due to missing parameters
+!   from the modules.
+! ==============================================================================
 
-!     use geoclaw_module
-!     use hurricane_module
-!     use multilayer_module
+    use amr_module, only: mcapa
+    
+    use geoclaw_module, only: g => grav, earth_radius, pi
+
+    use multilayer_module, only: num_layers, eigen_method, inundation_method
+    use multilayer_module, only: eigen_func, inundation_eigen_func
+    use multilayer_module, only: dry_tolerance, rho, aux_layer_index
 
     implicit none
 
@@ -50,7 +57,7 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
     ! Single layer solver storage
     double precision :: ql_sl(3),qr_sl(3)
     double precision :: aux1_sl(3,2),aux2_sl(3,2),aux3_sl(3,2)
-    double precision :: asdq_sl(meqn),bmasdq_sl(meqn),bpasdq_sl(meqn)
+    double precision :: asdq_sl(3),bmasdq_sl(3),bpasdq_sl(3)
     
     double precision :: ts(6),teig_vec(6,6)
     
@@ -59,14 +66,6 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
     double precision :: h(2),hu(2),hv(2),u(2),v(2),h_hat(2),gamma
     double precision :: alpha(4)
     
-    ! Common block parameters
-    real(kind=8) :: rho(num_layers),r
-    real(kind=8) :: g,R_earth,pi,capa_index
-    integer :: eigen_method, inundation_method
-    real(kind=8) :: dry_tolerance
-    
-    common /cparam/ rho,R_earth
-    
     ! Output array intializations
     bmasdq = 0.d0
     bpasdq = 0.d0
@@ -101,7 +100,8 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
                 b(2) = aux2(1,i-1)
                 b(1) = aux1(1,i-1)
                 
-                h_hat = aux2(7:8,i-1)
+                h_hat(1) = aux2(aux_layer_index,i-1)
+                h_hat(2) = aux2(aux_layer_index+1,i-1)
             ! Solving in the right grid cell (A^+ \Delta Q)
             else
                 h(j) = ql(layer_index + 1,i) / rho(j)
@@ -112,10 +112,10 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
                 b(2) = aux2(1,i)
                 b(1) = aux1(1,i)
                 
-                h_hat = aux2(7:8,i)
+                h_hat = aux2(aux_layer_index:aux_layer_index+1,i)
             endif
                 
-            if (h(j) < dry_tolerance) then
+            if (h(j) < dry_tolerance(j)) then
                 u(j) = 0.d0
                 v(j) = 0.d0
             else
@@ -124,41 +124,40 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
             endif
         enddo
         
-!         if (h(2) < dry_tolerance) then
-!             do m=4,6
-!                 if (asdq(i,m) /= 0.d0) then
-!                     print *,"asdq(i,4:6) > 0"
-!                     print *,ixy,imp,i
-!                     print *,h(1),h(2)
-!                     print *,hu(1),hu(2)
-!                     print *,(asdq(i,mw),mw=4,6)
-! !                     stop
-! !                     asdq(i,m) = 0.d0
-!                 endif
-!             enddo
-!         endif
-            
-        
         ! ====================================================================
         !  Check for dry states in the bottom layer, 
         !  This is if we are right next to a wall, use single layer solver
-        if ((h(2) < dry_tolerance)) then
+        if (h(1) < dry_tolerance(1) .or. h(2) < dry_tolerance(2)) then
             ! Storage for single layer rpt2
-            ql_sl = ql(1:3,i) / rho(1)
-            qr_sl = qr(1:3,i-1) / rho(1)
+            if (h(1) < dry_tolerance(1)) then
+                ql_sl = ql(1:3,i) / rho(1)
+                qr_sl = qr(1:3,i-1) / rho(1)
+                asdq_sl = asdq(1:3,i) / rho(1)
+            else if (h(2) < dry_tolerance(2)) then
+                ql_sl = ql(4:6,i) / rho(2)
+                qr_sl = qr(4:6,i-1) / rho(2)
+                asdq_sl = asdq(4:6,i) / rho(2)
+            else
+                print *, "Invalid dry-state found."
+                print *, "  h = ", h
+                stop
+            end if
+
             aux1_sl = aux1(1:3,i-1:i)
             aux2_sl = aux2(1:3,i-1:i)
             aux3_sl = aux3(1:3,i-1:i)
-            asdq_sl = asdq(1:3,i) / rho(1)
             
             ! Call solve
-            call rpt2_single_layer(ixy,ql_sl,qr_sl,aux1_sl,aux2_sl, &
-                                   aux3_sl,imp,asdq_sl,bmasdq_sl,bpasdq_sl)
+            call rpt2_single_layer(ixy,imp,ql_sl,qr_sl,aux1_sl,aux2_sl, &
+                                   aux3_sl,asdq_sl,bmasdq_sl,bpasdq_sl)
             
-            bmasdq(1:3,i) = bmasdq_sl * rho(1)
-            bmasdq(4:6,i) = 0.d0
-            bpasdq(1:3,i) = bpasdq_sl * rho(1)
-            bpasdq(4:6,i) = 0.d0
+            if (h(1) < dry_tolerance(1)) then
+                bmasdq(1:3,i) = bmasdq_sl * rho(1)
+                bpasdq(1:3,i) = bpasdq_sl * rho(1)
+            else if (h(2) < dry_tolerance(2)) then
+                bmasdq(4:6,i) = bmasdq_sl * rho(2)
+                bpasdq(4:6,i) = bpasdq_sl * rho(2)
+            end if
             cycle
         endif
         
@@ -166,17 +165,9 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
         ! Two-layers with no dry states in the vicinity
         ! Compute eigenvector matrix - Linearized system
         if (eigen_method == 1) then
-            call linearized_eigen(h_hat,h_hat,v,v,u,u,t_index,n_index,s,eig_vec)
-        else if (eigen_method == 2) then
-            call linearized_eigen(h,h,v,v,u,u,t_index,n_index,s,eig_vec)
-        else if (eigen_method == 3) then
-            call vel_diff_eigen(h,h,v,v,u,u,t_index,n_index,s,eig_vec)
-        else if (eigen_method == 4) then
-            call lapack_eigen(h,h,v,v,u,u,t_index,n_index,s,eig_vec)
+            call eigen_func(h_hat,h_hat,v,v,u,u,t_index,n_index,s,eig_vec)
         else
-            print "(a,i2,a)","Eigenstructure method ",eigen_method, &
-                  " requested not available."
-            stop 
+            call eigen_func(h,h,v,v,u,u,t_index,n_index,s,eig_vec)
         endif
 
         ! ====================================================================
@@ -208,17 +199,17 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
         endif
 
         ! Handle lat-long coordinate systems
-        if (capa_index > 0) then
+        if (mcapa > 0) then
             if (ixy == 2) then
-                dxdcp=(R_earth*pi/180.d0)
+                dxdcp=(earth_radius*pi/180.d0)
                 dxdcm = dxdcp
             else
                 if (imp == 1) then
-                    dxdcp = R_earth*pi*cos(aux3(3,i-1))/180.d0
-                    dxdcm = R_earth*pi*cos(aux1(3,i-1))/180.d0
+                    dxdcp = earth_radius*pi*cos(aux3(3,i-1))/180.d0
+                    dxdcm = earth_radius*pi*cos(aux1(3,i-1))/180.d0
                 else
-                    dxdcp = R_earth*pi*cos(aux3(3,i))/180.d0
-                    dxdcm = R_earth*pi*cos(aux1(3,i))/180.d0
+                    dxdcp = earth_radius*pi*cos(aux3(3,i))/180.d0
+                    dxdcm = earth_radius*pi*cos(aux1(3,i))/180.d0
                 endif
             endif
         else
@@ -246,9 +237,10 @@ subroutine rpt2(ixy,imp,maxm,meqn,mwaves,maux,mbc,mx,ql,qr,aux1,aux2,aux3,asdq,b
             endif
         enddo
     enddo
-    
+
 end subroutine rpt2
 
+
 subroutine rpt2_single_layer(ixy,imp,ql,qr,aux1,aux2,aux3,asdq,bmasdq,bpasdq)
 ! Single layer point-wise transverse Riemann solver using an einfeldt Jacobian
 ! Note that there have been some changes to variable definitions in this
@@ -256,55 +248,49 @@ subroutine rpt2_single_layer(ixy,imp,ql,qr,aux1,aux2,aux3,asdq,bmasdq,bpasdq)
 !
 ! Adapted from geoclaw 4-23-2011
 
-!       use geoclaw_module
+    use amr_module, only: mcapa
+
+    use geoclaw_module, only: g => grav, earth_radius, pi
+    use multilayer_module, only: num_layers, eigen_method, inundation_method
+    use multilayer_module, only: dry_tolerance
 
     implicit none
 
-    integer ixy,imp
-    
     integer, parameter :: meqn = 3
     integer, parameter :: mwaves = 3
 
-    double precision  ql(meqn) ! = ql(i,meqn)
-    double precision  qr(meqn) ! = qr(i-1,meqn)
-    double precision  asdq(meqn)
-    double precision  bmasdq(meqn)
-    double precision  bpasdq(meqn)
+    integer, intent(in) :: ixy,imp
+
+    real(kind=8), intent(in) :: ql(meqn) ! = ql(i,meqn)
+    real(kind=8), intent(in) :: qr(meqn) ! = qr(i-1,meqn)
+    real(kind=8), intent(in out) :: asdq(meqn)
+    real(kind=8), intent(in out) :: bmasdq(meqn)
+    real(kind=8), intent(in out) :: bpasdq(meqn)
     ! Since we need two values of aux, 1 = i-1 and 2 = i
-    double precision  aux1(2,3)
-    double precision  aux2(2,3)
-    double precision  aux3(2,3)
+    real(kind=8), intent(in) :: aux1(2,3)
+    real(kind=8), intent(in) :: aux2(2,3)
+    real(kind=8), intent(in) :: aux3(2,3)
 
-    double precision  s(3)
-    double precision  r(3,3)
-    double precision  beta(3)
-    double precision  tol,abs_tol
-    double precision  hl,hr,hul,hur,hvl,hvr,vl,vr,ul,ur,bl,br
-    double precision  uhat,vhat,hhat,roe1,roe3,s1,s2,s3,s1l,s3r
-    double precision  delf1,delf2,delf3,dxdcd,dxdcu
-    double precision  dxdcm,dxdcp,topo1,topo3,eta
+    real(kind=8) :: s(3)
+    real(kind=8) :: r(3,3)
+    real(kind=8) :: beta(3)
+    real(kind=8) :: abs_tol
+    real(kind=8) :: hl,hr,hul,hur,hvl,hvr,vl,vr,ul,ur,bl,br
+    real(kind=8) :: uhat,vhat,hhat,roe1,roe3,s1,s2,s3,s1l,s3r
+    real(kind=8) :: delf1,delf2,delf3,dxdcd,dxdcu
+    real(kind=8) :: dxdcm,dxdcp,topo1,topo3,eta,tol
 
-    integer m,mw,mu,mv
-
-    ! Common block parameters
-    real(kind=8) :: rho(num_layers),r
-    real(kind=8) :: g,R_earth,pi,capa_index
-    integer :: eigen_method, inundation_method
-    real(kind=8) :: dry_tolerance
+    integer ::  m,mw,mu,mv
     
-    common /cparam/ rho,R_earth
-    
-    
-    
-    tol=dry_tolerance
-    abs_tol=dry_tolerance
+    tol = dry_tolerance(1)
+    abs_tol = tol
 
     if (ixy.eq.1) then
-	   mu = 2
-	   mv = 3
+     mu = 2
+     mv = 3
     else
-	   mu = 3
-	   mv = 2
+     mu = 3
+     mv = 2
     endif
 
 
@@ -344,7 +330,7 @@ subroutine rpt2_single_layer(ixy,imp,ql,qr,aux1,aux2,aux3,asdq,bmasdq,bpasdq)
       dxdcp = 1.d0
       dxdcm = 1.d0
 
-       if (hl.le.dry_tolerance.and.hr.le.dry_tolerance) go to 90
+       if (hl.le.dry_tolerance(1).and.hr.le.dry_tolerance(1)) go to 90
 
        !check and see if cell that transverse waves are going in is high and dry
        if (imp.eq.1) then
@@ -358,17 +344,17 @@ subroutine rpt2_single_layer(ixy,imp,ql,qr,aux1,aux2,aux3,asdq,bmasdq,bpasdq)
        endif
        if (eta.lt.max(topo1,topo3)) go to 90
 
-      if (capa_index > 0) then
+      if (mcapa > 0) then
          if (ixy.eq.2) then
-            dxdcp=(R_earth*pi/180.d0)
+            dxdcp=(earth_radius*pi/180.d0)
             dxdcm = dxdcp
          else
             if (imp.eq.1) then
-               dxdcp = R_earth*pi*cos(aux3(1,3))/180.d0
-               dxdcm = R_earth*pi*cos(aux1(1,3))/180.d0
+               dxdcp = earth_radius*pi*cos(aux3(1,3))/180.d0
+               dxdcm = earth_radius*pi*cos(aux1(1,3))/180.d0
             else
-               dxdcp = R_earth*pi*cos(aux3(2,3))/180.d0
-               dxdcm = R_earth*pi*cos(aux1(2,3))/180.d0
+               dxdcp = earth_radius*pi*cos(aux3(2,3))/180.d0
+               dxdcm = earth_radius*pi*cos(aux1(2,3))/180.d0
             endif
          endif
       endif
@@ -436,167 +422,7 @@ subroutine rpt2_single_layer(ixy,imp,ql,qr,aux1,aux2,aux3,asdq,bmasdq,bpasdq)
                  bpasdq(mv)=bpasdq(mv)+ dxdcp*s(mw)*beta(mw)*r(3,mw)
                endif
             enddo
-!========================================================================
 
-! 
-!     use geoclaw_module
-! 
-!     implicit none
-! 
-!     ! Position input
-!     integer, intent(in) :: ixy,imp,mwaves,meqn
-! 
-!     ! State Input/Output
-!     double precision, intent(in) :: q_l(meqn),q_r(meqn)
-!     double precision, intent(in) :: asdq(meqn),aux1(2,3),aux2(2,3),aux3(2,3)
-!     double precision, intent(out) :: bmasdq(meqn),bpasdq(meqn)
-! 
-!     ! Local storage
-!     integer :: m,mw,mu,mv
-!     double precision :: s(3)
-!     double precision :: r(3,3)
-!     double precision :: beta(3)
-!     double precision :: g,tol,abs_tol
-!     double precision :: hl,hr,hul,hur,hvl,hvr,vl,vr,ul,ur,bl,br
-!     double precision :: uhat,vhat,hhat,roe1,roe3,s1,s2,s3,s1l,s3r
-!     double precision :: delf1,delf2,delf3,dxdcd,dxdcu
-!     double precision :: dxdcm,dxdcp,topo1,topo3,eta
-! 
-!     ! Constants
-!     g=grav
-!     tol=dry_tolerance
-!     abs_tol=dry_tolerance
-!     
-!     ! Initializations
-!     s = 0.d0
-!     beta = 0.d0
-!     r = 0.d0
-! 
-!     if (ixy.eq.1) then
-!         mu = 2
-!         mv = 3
-!     else
-!         mu = 3
-!         mv = 2
-!     endif
-! 
-!     ! Extract states from q
-!     hl=q_l(1)
-!     hr=q_r(1)
-!     hul=q_l(mu)
-!     hur=q_r(mu)
-!     hvl=q_l(mv)
-!     hvr=q_r(mv)
-! 
-! !===========determine velocity from momentum===========================
-!     if (hl.lt.abs_tol) then
-!         hl=0.d0
-!         ul=0.d0
-!         vl=0.d0
-!     else
-!         ul=hul/hl
-!         vl=hvl/hl
-!     endif
-! 
-!     if (hr.lt.abs_tol) then
-!         hr=0.d0
-!         ur=0.d0
-!         vr=0.d0
-!     else
-!         ur=hur/hr
-!         vr=hvr/hr
-!     endif
-! 
-!     dxdcp = 1.d0
-!     dxdcm = 1.d0
-! 
-!     if (hl.le.dry_tolerance.and.hr.le.dry_tolerance) go to 90
-! 
-!     ! check and see if cell that transverse waves are going in is high and dry
-!     if (imp.eq.1) then
-!         eta = q_l(1) + aux2(1,1)
-!         topo1 = aux1(1,1)
-!         topo3 = aux3(1,1)
-!     else
-!         eta = q_r(1) + aux2(2,1)
-!         topo1 = aux1(2,1)
-!         topo3 = aux3(2,1)
-!     endif
-!     if (eta.lt.max(topo1,topo3)) go to 90
-! 
-!     if (icoordsys.eq.2) then
-!         if (ixy.eq.2) then
-!             dxdcp=(R_earth*pi/180.d0)
-!             dxdcm = dxdcp
-!         else
-!             if (imp.eq.1) then
-!                 dxdcp = R_earth*pi*cos(aux3(1,3))/180.d0
-!                 dxdcm = R_earth*pi*cos(aux1(1,3))/180.d0
-!             else
-!                 dxdcp = R_earth*pi*cos(aux3(2,3))/180.d0
-!                 dxdcm = R_earth*pi*cos(aux1(2,3))/180.d0
-!             endif
-!         endif
-!     endif
-! 
-! ! =====Determine some speeds necessary for the Jacobian=================
-!     vhat=(vr*dsqrt(hr))/(dsqrt(hr)+dsqrt(hl)) + (vl*dsqrt(hl))/(dsqrt(hr)+dsqrt(hl))
-! 
-!     uhat=(ur*dsqrt(hr))/(dsqrt(hr)+dsqrt(hl)) + (ul*dsqrt(hl))/(dsqrt(hr)+dsqrt(hl))
-!     hhat=(hr+hl)/2.d0
-! 
-!     roe1=vhat-dsqrt(g*hhat)
-!     roe3=vhat+dsqrt(g*hhat)
-! 
-!     s1l=vl-dsqrt(g*hl)
-!     s3r=vr+dsqrt(g*hr)
-! 
-!     s1=dmin1(roe1,s1l)
-!     s3=dmax1(roe3,s3r)
-! 
-!     s2=0.5d0*(s1+s3)
-! 
-!     s(1)=s1
-!     s(2)=s2
-!     s(3)=s3
-! ! =======================Determine asdq decomposition (beta)============
-!     delf1=asdq(1)
-!     delf2=asdq(mu)
-!     delf3=asdq(mv)
-! 
-!     beta(1) = (s3*delf1/(s3-s1))-(delf3/(s3-s1))
-!     beta(2) = -s2*delf1 + delf2
-!     beta(3) = (delf3/(s3-s1))-(s1*delf1/(s3-s1))
-! ! ======================End =================================================
-! 
-! ! =====================Set-up eigenvectors===================================
-!     r(1,1) = 1.d0
-!     r(2,1) = s2
-!     r(3,1) = s1
-! 
-!     r(1,2) = 0.d0
-!     r(2,2) = 1.d0
-!     r(3,2) = 0.d0
-! 
-!     r(1,3) = 1.d0
-!     r(2,3) = s2
-!     r(3,3) = s3
-! !============================================================================
-! 90  continue
-! !============= compute fluctuations==========================================
-!     
-!     bmasdq = 0.0d0
-!     bpasdq = 0.0d0
-!     do  mw=1,3
-!         if (s(mw).lt.0.d0) then
-!             bmasdq(1)  = bmasdq(1)  + dxdcm*s(mw)*beta(mw)*r(1,mw)
-!             bmasdq(mu) = bmasdq(mu) + dxdcm*s(mw)*beta(mw)*r(2,mw)
-!             bmasdq(mv) = bmasdq(mv) + dxdcm*s(mw)*beta(mw)*r(3,mw)
-!         elseif (s(mw).gt.0.d0) then
-!             bpasdq(1)  = bpasdq(1)  + dxdcp*s(mw)*beta(mw)*r(1,mw)
-!             bpasdq(mu) = bpasdq(mu) + dxdcp*s(mw)*beta(mw)*r(2,mw)
-!             bpasdq(mv) = bpasdq(mv) + dxdcp*s(mw)*beta(mw)*r(3,mw)
-!         endif
-!     enddo
 
-end subroutine rpt2_single_layer
\ No newline at end of file
+end subroutine rpt2_single_layer
+
